/* FILE: convert.c             Copyright (c), Symmetric Research, 1998
 *
 * Floating point conversion between DSP32C binary, IEEE, and ASCII.
 *
 *      IeeeToDspf      IEEE -> DSP32C
 *      DspfToIeee      DSP32C -> IEEE
 *      IeeeToA         IEEE -> ASCII string
 *      DspfToA         DSP32C -> ASCII string
 *      AToIeee         ASCII string -> IEEE
 *      AToDspf         ASCII string -> DSP32C
 *
 *
 *
 * * FLOATING POINT CONVERSIONS ON THE DSP32C
 *
 * Floating point conversions between IEEE and DSP32C binary is much
 * better done by the DSP32C itself than by these functions.  Use these
 * functions to do only those conversions that cannot be done on the board
 * itself.
 *
 * * POSSIBLE OVERFLOW
 *
 * The version of fscan below can overflow on numbers with large
 * exponents, such as 1.0E+45, causing a math exception in the C runtime
 * library and AToIeee and AToDspf to crash.
 *
 *
 *
 */

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

#include "dsptools.h"






FUNCTYPE( VOID) DSP32C_IeeeToDspf( float *x) {

        long sign, m; unsigned long e;
        

        /* Special case 0. */

        if ( *(unsigned long *)x == 0L )

                return;


        /* Otherwise shuffle bits. */
        
        sign = *(unsigned long *)x & 0x80000000L;

            m = *(unsigned long *)x & 0x007FFFFF;
                
                e = ( *(unsigned long *)x >>23 )&0xFF;



        if ( e < 0xFF )

                e += 1;


        if ( sign ) {

                if ( m )

                        m = -m & 0x007FFFFF;

                else

                        e -= 1;
                }


        *(unsigned long *)x = sign | m<<8 | e;



}
FUNCTYPE( VOID) DSP32C_DspfToIeee( float *x) {

        long sign, m; unsigned long e;
        


        /* Special case 0. */
        
        if ( *(unsigned long *)x == 0L )

                return;
        

        /* Otherwise shuffle bits. */

        sign = *(unsigned long *)x & 0x80000000L;

            m = ( *(unsigned long *)x & 0x7FFFFF00L )>>8;

                e = *(unsigned long *)x & 0xFF;


        if ( e > 0 )

                e -= 1;
        

        if ( sign ) {
                
                if ( m )

                        m = -m & 0x007FFFFF;

                else

                        e += 1;

                }


        *(unsigned long *)x = sign | m | (e<<23);



}
FUNCTYPE( PCHAR) DSP32C_IeeeToA( float *x) {

        static char buffer[32];

          union { float f; unsigned long l; } value;

            float logx; double mantissa, exp;
            
              int msign, esign, e;



        /* Make a binary copy of the value of x. */

        value.l = *(unsigned long *)x;



        /* Special case 0. */

        if ( value.l == 0L )

                strcpy( buffer, "+0.000000E+00");


        /* Special case exp = 255. */

        else if ( (value.l & 0x7F800000L) == 0x7F800000L )

                strcpy( buffer, "infinity     ");
        



        /* Otherwise do the conversion. */

        else {

                if ( value.f < 0.0 )

                      { value.f = - value.f; msign = '-'; }

                  else
                        msign = '+';


                
                logx = (float)log10( value.f); /* round back to float */
                
                    mantissa = pow( 10.0, modf( logx, &exp));



                if ( (e = (int)exp) < 0 )

                        { e = -e; esign = '-'; }

                  else
                        esign = '+';



                sprintf( buffer, "%c%7.6fE%c%02d", msign, mantissa, esign, e);

                }



        return( buffer);


}
FUNCTYPE( PCHAR) DSP32C_DspfToA( float *x) {

        union { float f; unsigned long l; } value;



        /* Make a binary copy of the value of x. */

        value.l = *(unsigned long *)x;



        /* Do the conversion. */

        DSP32C_DspfToIeee( &value.f);

            return( DSP32C_IeeeToA( &value.f));



}
static char __far *space( char __far *p) {


        /* Skip over any white space. */

        while ( *p == ' ' || *p == '\t' )

                p++;


        return( p);


}
static int fscan( char __far *line, float *x) {

        char __far *save; int sign; double mantissa, f; int exp;


        
        /* Scan for [+,-]mmm[.[mmmm]][{E,e}[+,-]ee] */

        save = line;
        


        /* Skip over any leading space. */

        line = space( line);



        /* Get an optional sign. */

        if ( *line == '+' )

                { sign = 1; line++; }
        
        else if ( *line == '-' )

                { sign = -1; line++; }

        else
                sign = 1;
        


        /* Skip over any space before the mantissa. */

        line = space( line);



        /* Get the mantissa. */

        if ( isdigit( *line) ) {


                for ( mantissa = 0.0 ; isdigit( *line) ; )

                        mantissa = 10*mantissa + (*line++ - '0');
        

                if ( *line == '.' ) {

                        line++;

                        if ( isdigit( *line) ) {

                                for ( f = .1 ; isdigit( *line) ; f *= .1 )

                                        mantissa += (*line++ - '0')*f;

                                }
                        }

                mantissa *= sign;

                }

        else

              return( 0);



        /* Get an optional exponent. */

        if ( *line == 'E' || *line == 'e' ) {


                line++;

                
                /* Get the sign of the exponent. */

                if ( *line == '+' )

                        { sign = 1; line++; }
                
                else if ( *line == '-' )

                        { sign = -1; line++; }

                else
                        sign = 1;
                
                
                /* Get the integer part of the exponent. */

                if ( isdigit( *line) ) {

                        for ( exp = 0 ; isdigit( *line) ; )

                                exp = 10*exp + (*line++ - '0');

                        exp *= sign;

                        }
                else
                        return( 0);

                }


        else

                exp = 0;




        /* Return the float and number of bytes scanned. */

        *x = (float)( mantissa * pow( 10.0, (double)exp));


            return( /* nbytes = */ line-save );


}
FUNCTYPE( INT) DSP32C_AToDspf( char __far *line, float *x) {
        
        int nbytes;


        if ( (nbytes = fscan( line, x)) != 0 )
                
                DSP32C_IeeeToDspf( x);

        return( nbytes);

}
FUNCTYPE( INT) DSP32C_AToIeee( char __far *line, float *x) {
        
        return( /* nbytes = */ fscan( line, x));

}
