/* FILE: dspio.c                Copyright (c), Symmetric Research, 1998-1999
 *
 * This is the DOS version of the low level DSP32C support functions.  
 * The function calls in this file agree with those in the Win9x and WinNT
 * dspio.c.  Applications that work with those files will also work with
 * this one without modification.
 *
 * These functions provide the low level interface to the SR DSP32C
 * boards.  A common interface is possible because the DSP32C PIO is
 * used for board communication.  For technical specs on the DSP32C
 * PIO please refer to the AT&T DSP32C Information manual.
 * 
 * For the DSPMUL multiple cpu board, there is an on board register for
 * cpu selection in addition to the standard PIO interface.  The
 * function select_cpu is for poking this register.
 * 
 * These functions are used consistently throughout the SR supplied
 * software.  The functions in dsptools.c, dspmon.c, etc are built on
 * these lower level stand alone functions in dspio.c.
 * 
 * For the command line options required to correctly compile, see the
 * makefile in this directory.  For the protos of functions in dspio.c
 * see the header file dsptools.h in \sr\dsp32c\include.
 * 
 * 
 * 
 * 
 * NOTES:
 * 
 * * AVOIDANCE OF THE C RUNTIME LIBRARY:
 * 
 * Enough functionality has been included in this file to support the
 * DSPMATH library without using dsptools.c and its references to the C
 * runtime library.  C runtime references cause mixed language
 * programmers trouble at link time.
 * 
 * This file is a bare mimimum of what is needed to work with the board.
 * When augmented with dsptools.c it becomes the full DSPTOOLS library.
 * If you have an application which shouldn't call the C runtime, consider
 * compiling and linking to only to this file.
 * 
 *
 * * INLINE ASSEMBLER:
 * 
 * These functions use inline assembler when necessary to gain access to
 * 80x86 instructions not available from C.  In particular the rep insw
 * and rep outsw 80286 instructions are used to pump data through the
 * PIO port.  Be sure to compile with 286 instructions enabled.
 * 
 * 
 * * BASEIO SETS THE BASE ADDRESS FOR ALL OF DSPTOOLS:
 * 
 * DSPTOOLS uses the global variable baseio as the PC IO space base
 * address for the DSP32C PIO port.  In order to work correctly, the
 * value of baseio MUST agree with the DIP switch settings on your board.
 * 
 * The default value of baseio and corresponding DIP switch is assumed
 * to be 0x280, as defined by the DEF_BASEIO constant below.  The PIO
 * base address DIP switch has been set at the factory to agree with
 * this default value.  If you forget what the default DIP switch
 * settings are, refer to the circuit diagram for your board in the SR
 * user manual.
 * 
 * To run at values of baseio other than the default, you can set the
 * DOS DSP32C_BASEIO environment variable before running your program,
 * or explicity set the value of baseio in your program before calling
 * these functions.  If you work at a non default baseio values, make
 * sure your DIP switch agrees with the new value.
 * 
 * Finally, note that several boards can be run in one machine at the
 * same time by setting their DIP switches to different addresses, and
 * then setting baseio to the value for a respective board before
 * calling DSPTOOLS.  By consistently using DSPTOOLS to communicate with
 * the DSP32C boards, you guarantee your programs will be able to work
 * easily with non default base addresses.
 * 
 * 
 * * BASESEL SETS THE BASE ADDRESS FOR DSPMUL SELECTION REGISTER:
 * 
 * Like the baseio variable, basesel has the current base address of the
 * DSPMUL multiple cpu board selection register.  In order to work
 * correctly, the value of basesel MUST agree with the DIP switch
 * settings on the DSPMUL board.
 * 
 * The default value of basesel is assumed to be 0x2A0, and like the
 * baseio variable, this can be overidden by either setting the DOS
 * environment variable DSP32C_BASESEL or explicity setting basesel in
 * your programs.
 * 
 * 
 * * LOW LEVEL HALT AND RESET OF THE DSP32C:
 * 
 * The low level halt and reset of the DSP32C requires care.  The chip
 * is fairly sensitive to having its PIO PCR bits initialized in the
 * correct order, and the following covers some of the important points
 * to keep in mind when reading the functions below.
 * 
 * Reset occurs ONLY ON the 0 to 1 transition of PCR[0] or the RESETN
 * pin.  Setting PCR[0] to 0 halts, but does not cause a reset.
 * 
 * After ANY reset, software or hardware, the DSP32C PIO port is in the
 * 8 bit DSP32 compatible mode, and MUST be reconfigured for the 16 bit
 * DSP32C DMA mode to work properly with the hardware on the coprocessor
 * board.
 * 
 * The function DSP32C_Init below halts the system, but does not cause a
 * reset.  To make sure the PC can communicate with the board, the PIO
 * port is also programmed for the 16 bit DMA mode.  After calling
 * DSP32C_Init the system is in a known state and ready for downloading
 * program and data before beginning execution.
 * 
 * The function go forces the processor to initiate its reset sequence,
 * and begin execution from location 0.  After the reset, the PIO is
 * again in its 8 bit mode, and the port is once more reconfigured for
 * the 16 bit DMA mode.
 * 
 * Table 6-3 in the AT&T DSP32C Information manual covers the sequence
 * of programming steps required to take the PIO from the DSP32 8 bit
 * compatible mode after a reset to the DSP32C 16 bit DMA mode the
 * DSPTOOLS library uses.
 * 
 * Note that after power up, the DSP32C coprocessor board has executed a
 * hardware reset and is running.  The processor will be executing
 * garbage until the board is downloaded with program and data through
 * the PIO port.
 * 
 * 
 *
 *
 */

#include "dspio.h"







/* High and low word inline assembler helper macros:
 *
 * These two macros are for addressing the high and low words of a 32 bit
 * long integer or far/huge pointer.
 *
 *
 */

#define HW( p)          WORD PTR p[2]
#define LW( p)          WORD PTR p[0]





/* RELEASE REV INFO:
 *
 * Allow the user to determine the library rev they are working with.  See
 * dspio.h for more details.
 *
 */

VARTYPE( INT ) DSP32C_Rev = DSP32C_REV;

FUNCTYPE( INT ) DSP32C_GetRev( void ) {

        /* Return the library rev number. */

        return( DSP32C_Rev );
}



/* DSP32C base addresses and PIO register offsets:
 *
 * These global variables and defines have the current base addresses and
 * layout of the PC to DSP32C board addressing.
 *
 * The helper functions read_pio and write_pio are used in non critical
 * code below for accessing the PIO registers.
 *
 *
 */

VARTYPE( INT) __near DSP32C_BaseIo   = 0x0000;   /* PIO base io address */
VARTYPE( INT) __near DSP32C_BaseSel  = 0x0000;   /* DSPMUL cpu select addr */



#define DEFAULT_BASEIO      0x280    /* default PC base address           */
#define DEFAULT_BASESEL     0x2A0    /* default PC base address           */

#define PCRl            (0x07*2)    /* PIO DSP32 mode PCR low            */
#define PCRh            (0x0A*2)    /* PIO DSP32C 8 bit mode PCR high    */

#define PAR             (0x00*2)    /* PIO DSP32C 16 bit mode registers  */
#define PDR             (0x02*2)
#define EMR             (0x04*2)
#define ESR             (0x06*2)
#define PCR             (0x07*2)
#define PIR             (0x08*2)
#define PARE            (0x0B*2)
#define PDR2            (0x0C*2)



static int ReadPio( unsigned reg_offset) {

        int value;


        /* Read a PIO register. */

        _asm {

                mov     dx,DSP32C_BaseIo
                add     dx,reg_offset
                in      ax,dx
                mov     value,ax

                }


        return( value);


}
static void WritePio( int reg_offset, int value) {


        /* Write a PIO register. */

        _asm {

                mov     dx,DSP32C_BaseIo
                add     dx,reg_offset
                mov     ax,value
                out     dx,ax

                }

}









/* Device driver functions:
 *
 * These functions are NOPs provided for compatibility with systems that
 * need to open and close the communication connection between the DLL 
 * and a device driver.
 *
 */

FUNCTYPE( INT) DSP32C_Open( void) {
        return( 1 );
}
FUNCTYPE( INT) DSP32C_Close( void) {
        return( 1 );
}





/* Functions for changing the values of baseio and basesel:
 *
 * These functions allow the baseio and basesel global variables to be
 * changed through functions rather than directly.
 *
 *
 *
 *
 */

FUNCTYPE( VOID) DSP32C_SetBaseIo( unsigned new_baseio) {

        DSP32C_BaseIo = new_baseio;

}
FUNCTYPE( VOID) DSP32C_SetBaseSel( unsigned new_basesel) {

        DSP32C_BaseSel = new_basesel;

}
FUNCTYPE( INT) DSP32C_GetBaseIo( void) {

        return( DSP32C_BaseIo);

}
FUNCTYPE( INT) DSP32C_GetBaseSel( void) {

        return( DSP32C_BaseSel);

}









/* DOS environment space getenv_DSP32C helper:
 *
 * This helper function is called by DSP32C_Init and DSP32C_SelectCpu below.  
 * It checks DOS environment space for the specified env variable name and
 * value.
 *
 * If the specified env variable is found, up to 4 trailing ASCII hex
 * digits are converted to a binary integer and returned.  If the env
 * variable cannot be found or its ASCII value cannot be converted to
 * binary, zero is returned.
 *
 * Rather than using the standard C runtime getenv function, this
 * function is included here so dspio.c can run standalone without
 * any C runtime help.
 *
 *
 */

static char __far *skipspace( char __far *p) {


        /* Skip over any white space. */

        while ( *p == ' ' || *p == '\t' )

                p++;


        return( p);


}
static char __far *strequ( char __far *src, char *dest) {


        /* Compare the src against the dest string. */

        while ( *src == *dest )

                { src++; dest++; }


        return( *dest == '\0' ? src : (char __far *)0 );



}
static unsigned getenv_DSP32C( char *var_name) {

        char __far *env, __far *p; unsigned n, ndigits, d, c;



        /* Point to the DOS environment space. */

        _asm {

                mov     ah,0x62              /* DOS func 0x62 = get PSP */
                int     0x21

                mov     es,bx                /* bx = psp segment */
                mov     bx,0x2C

                mov     bx,es:[bx]           /* bx = env segment */

                mov     HW( env),bx
                mov     LW( env),0x0000

                }



        /* Scan for a match with the specified variable name. */

        while ( *env ) {


                /* Check for a match. */

                if ( (p = strequ( env, var_name)) != (char __far *)0 )

                        { env = p; break; }



                /* If no match, advance to next env variable. */

                while ( *env )

                        env++;

                env++;

                }




        /* If at end of env space with no match, return failure. */

        if ( *env == '\0' )

                return( 0);




        /* Scan for an " = " sign. */

        env = skipspace( env);


        if ( *env == '=' )

                env++;
        else

                return( 0);




        /* Scan for an ASCII hex number. */

        env = skipspace( env);


        for ( n = 0 , ndigits = 0 ; ndigits < 4 ; ndigits++ ) {

            c = *env;

                if ( '0' <= c && c <= '9' )

                        d = c - '0';

                else if ( 'a' <= c && c <= 'f' )

                        d = c - 'a' + 0xA;

                else if ( 'A' <= c && c <= 'F' )

                        d = c - 'A' + 0xA;

                else
                        break;


                    n = (n<<4) + d; env++;

                }




        /* Check for the end of the env variable and return. */

        env = skipspace( env);

            return( *env == '\0' ? n : 0 );

}












/* Processor and library initialization:
 *
 * This function initializes the DSP32C board and DSPTOOLS library.  It
 * should be called at the beginning of every program using the system
 * to make sure a board is installed, and to place it in a known state
 * ready for downloading.  The function returns a 1 if a board is in the
 * system and succesfully intialized, 0 if not.
 *
 * As part of the initialization, DSP32C_Init calls getenv_DSP32C above to
 * see if the DOS environment variable DSP32C_BASEIO has been set by the
 * user to change the variable baseio from the 0x280 default value.
 *
 * After successful intialization the DSP32C is left halted, but the PIO
 * port is set up in its 16 bit DMA autoincrement mode for communication
 * and downloading.  Downloading a program and calling go will begin
 * execution.
 *
 * For the DSPMUL multiple cpu board, users should call DSP32C_SelectCpu
 * BEFORE calling DSP32C_Init to select the cpu to initialize.
 *
 *
 *
 */

#define BASEIO_ENV_NAME    "DSP32C_BASEIO"


FUNCTYPE( INT) DSP32C_Init( void) {


        /* Initialize baseio.
         *
         * Note that this code allows the user to override either a DOS
         * environment space value or the normal default with a nonzero
         * value they have set before calling DSP32C_Init.
         *
         * This can be useful for multiple board installations where the
         * user must call DSP32C_Init with baseio set to the IO address for
         * each board.
         *
         *
         */

#define _SEARCHENV

#ifdef _SEARCHENV

        if ( DSP32C_BaseIo == 0x0000 ) {


                /* Check DOS environment space for an override. */

                DSP32C_BaseIo = getenv_DSP32C( BASEIO_ENV_NAME);



                /* If not found, use the default value. */

                if ( DSP32C_BaseIo == 0x0000 )

                        DSP32C_BaseIo = DEFAULT_BASEIO;

                }


#else
        DSP32C_BaseIo  = DEFAULT_BASEIO;
#endif


        /* Halt the DSP32C and configure the PIO port.
         *
         * NOTE:
         *
         * After power up or software reset, the DSP32C's PIO port is in
         * the DSP32 8 bit mode and must be reprogrammed for the full DSP32C
         * 16 bit auto increment dma mode.  See Table 6-3 on page 6-9 of
         * the AT&T manual for the sequence of steps.
         *
         * The DSP32C is left halted after the PIO port is programmed so
         * it can be downloaded, but won't be executing.
         *
         *
         */

        WritePio( PCRl, 0x02);         /* set REGMAP for DSP32C PAB map */
        WritePio( PCRh, 0x02);         /* set PIO16 for 16 bit interface */

            WritePio( PCR, 0x21A);     /* set DMA and AUTO */




        /* Do a presence test, checking the PCR register value. */

        if ( (ReadPio( PCR) & 0x71F) != 0x21A )

                return( 0);




        /* Return success. */

        return( 1);


}










/* Execution control:
 *
 * These functions control the execution state of DSP32C.
 *
 * Leaving the system halted reduces power consumption when the system
 * is not doing useful work.
 *
 *
 */

FUNCTYPE( VOID) DSP32C_Go( void) {


        /* Reset the DSP32C and start execution from location 0.
         *
         * NOTE:
         *
         * As with DSP32C_Init, the PIO port must be reprogrammed for the 16
         * bit DMA mode after reset.
         *
         *
         */


        /* Toggle PCR[0], initiating reset and execution. */

        WritePio( PCRl, 0x00);
        WritePio( PCRl, 0x01);



        /* Reprogram the PIO port WHILE executing. */

        WritePio( PCRl, 0x03);         /* set REGMAP for DSP32C PAB map */
        WritePio( PCRh, 0x02);         /* set PIO16 for 16 bit interface */

            WritePio( PCR, 0x21B);     /* set DMA and AUTO */



}
FUNCTYPE( VOID) DSP32C_Halt( void) {

        int value;


        /* Halt the DSP32C, clearing PCR[0]. */

        value = ReadPio( PCR);

            value &= 0xFFFE;

               WritePio( PCR, value);


}










/* PIO registers functions:
 *
 * These function read and write the DSP32C PIO registers.
 *
 *
 */

FUNCTYPE( INT) DSP32C_Pcr( void) {

        return( ReadPio( PCR));

}
FUNCTYPE( INT) DSP32C_Pir( void) {

        return( ReadPio( PIR));

}
FUNCTYPE( INT) DSP32C_Esr( void) {

        return( ReadPio( ESR));

}
FUNCTYPE( INT) DSP32C_Emr( void) {

        return( ReadPio( EMR));

}
FUNCTYPE( VOID) DSP32C_SetEni( unsigned newstate) {

        unsigned value;


        /* Set the value of the PCR[2] ENI bit.
         *
         * This bit enables or disables the PIF pin which is connected
         * to the PC interrupt header on the board.  Newstate = 1 enables
         * the pin, newstate = 0 disables it.
         *
         * Note that the eni bit is automatically cleared by a reset of the
         * DSP32C, and must be reenabled after DSP32C_Init or go if interrupts
         * are being used.
         *
         *
         */

        value = ReadPio( PCR);


            value &= ~0x0004;

                 if ( newstate == 1 ) value |= 0x0004;


        WritePio( PCR, value);


}
FUNCTYPE( VOID) DSP32C_SetPir( unsigned value) {


        /* Set the PIR register to the specified value from the PC. */

        WritePio( PIR, value);

}











/* Data and program transfer:
 *
 * These functions exchange data and hexified programs with the DSP32C
 * board.  Also see dsptools.c for additional downloaders.
 *
 *
 * NOTES:
 *
 * The order in which PARE and PAR are set in these functions is VERY
 * important for their proper operation.  PARE must be written BEFORE
 * PAR.  Refer to Table 6-7 in the AT&T DSP32C Information manual for
 * the sequence of events in a PIO DMA transfer.
 *
 * The functions get_32C and put_32C DO NOT normalize their pointers to
 * PC data on entry.  While sometimes convenient in 80x86 real mode,
 * pointer normaliztion is not acceptable for protected mode
 * applications.
 * 
 * If get_32C or put_32C are moving LESS than 64K of data, they assume
 * all of the data is accessable with 16 bit offset addressing.  Objects
 * spanning segment boundaries are not acceptable.
 * 
 * If get_32C or put_32C are moving MORE than 64K of data, the pointer
 * to the PC data must have an offset part of 0 in order to work
 * correctly.  Fortunately the DOS huge allocator func 0x48 always
 * returns an allocated object with 0 offset and is compatible with
 * this assumtion.
 * 
 *
 *
 */

FUNCTYPE( VOID) DSP32C_Get( unsigned long src, void __huge *dest, unsigned long nbytes) {


        /* Read data from the DSP32C memory space, where:
         *
         *      src  = DSP32C 24 bit source address
         *     dest  = normalized PC segment:offset destination address
         *
         *
         */



        /* Set up for the transfer. */

        _asm {

                mov     dx,DSP32C_BaseIo        /* (PARE,PAR) -> src */
                add     dx,PARE
                mov     ax,HW( src)
                out     dx,ax

                mov     dx,DSP32C_BaseIo
                add     dx,PAR
                mov     ax,LW( src)
                out     dx,ax

                cld                             /* direction = up */

                }



        /* Loop, moving block of 0x10000 bytes = 0x8000 words. */

        while ( nbytes >= 0x10000L ) {

                _asm {

                        mov     dx,DSP32C_BaseIo  /* perform rep insw */
                        add     dx,PDR

                        mov     cx,0x8000

                        les     di,dest           /* es:[di] -> dest */
                        rep     insw

                        }


                /* Increment on to the next block. */

                ((unsigned char __huge *)dest) += 0x10000L; nbytes -= 0x10000L;

                }



        /* Clean up, moving any remaining bytes. */

        _asm {

                mov     dx,DSP32C_BaseIo        /* perform rep insw */
                add     dx,PDR

                mov     cx,LW( nbytes)          /* round nbytes to nwords */
                shr     cx,1
                adc     cx,0

                les     di,dest                 /* es:[di] -> dest */
                rep     insw

                }



}
FUNCTYPE( VOID) DSP32C_Put( void __huge *src, unsigned long dest, unsigned long nbytes) {


        /* Write data to the DSP32C memory space, where:
         *
         *      src  = normalized PC segment:offset source address
         *     dest  = DSP32C 24 bit destination address
         *
         *
         */



        /* Set up for the transfer. */

        _asm {

                mov     dx,DSP32C_BaseIo        /* (PARE,PAR) = dest */
                add     dx,PARE
                mov     ax,HW( dest)
                out     dx,ax

                mov     dx,DSP32C_BaseIo
                add     dx,PAR
                mov     ax,LW( dest)
                out     dx,ax

                cld                             /* direction = up */

                }



        /* Loop, moving blocks of 0x10000 bytes = 0x8000 words. */

        while ( nbytes >= 0x10000L ) {

                _asm {

                        mov     dx,DSP32C_BaseIo  /* perform rep outsw */
                        add     dx,PDR

                        mov     cx,0x8000         /* round nbytes to nwords */

                        push    ds                /* save ds ... */

                        lds     si,src            /* ds:[si] -> src */
                        rep     outsw

                        pop     ds                /* restore ds ... */

                        }


                /* Increment on to the next block. */

                ((unsigned char __huge *)src) += 0x10000L; nbytes -= 0x10000L;

                }



        /* Clean up, moving any remaining bytes. */

        _asm {

                mov     dx,DSP32C_BaseIo        /* perform rep outsw */
                add     dx,PDR

                mov     cx,LW( nbytes)          /* round nbytes to nwords */
                shr     cx,1
                adc     cx,0

                push    ds                      /* save ds ... */

                lds     si,src                  /* ds:[si] -> src */
                rep     outsw

                pop     ds                      /* restore ds ... */

                }




}
FUNCTYPE( VOID) DSP32C_PutLong( unsigned long value, unsigned long addr) {

        unsigned save, temp;


        /* This function writes a single long value to the DSP32C using
         * the 32 bit DMA mode of the PIO port.  The 16 bit PIO DMA autoinc
         * mode generally used by DSPTOOLS is saved and restored on
         * entry and exit.
         * 
         * This function is useful because even though the PIO port is
         * only 16 bits wide, it holds off writing until it has a full 32
         * bits.  This allows the PC to poke instructions and data onto
         * the board while competing with DSP32C execution, and not risk
         * fetching problems as in the case of two sequential 16 bit
         * writes.
         * 
         * The disadvantage of using this function is that it cannot use
         * the outsw instruction like put_32C.  In general, this function
         * should only be used for specialized applications, like breakpoint
         * execution in the DSPMON monitor.
         * 
         * As with the (PAR,PARE) registers, the order in which (PDR2,PDR)
         * is written is VERY important.  See the AT&T Information manual
         * for details.
         *
         *
         */



        /* Save the current PCR value. */

        save = ReadPio( PCR);



        /* Program the PCR for a 32 bit DMA transfer. */

        temp = save;

          temp &= 0x0465;               /* mask off bits not to change */
          temp |= 0x031A;               /* set the 32 bit DMA mode */

          WritePio( PCR, temp);



        /* Transfer the long value. */

        _asm {

                mov     dx,DSP32C_BaseIo        /* (PARE,PAR) -> src */
                add     dx,PARE
                mov     ax,HW( addr)
                out     dx,ax

                mov     dx,DSP32C_BaseIo
                add     dx,PAR
                mov     ax,LW( addr)
                out     dx,ax


                /* Write the data through (PDR2,PDR). */

                mov     dx,DSP32C_BaseIo
                add     dx,PDR2
                mov     ax,LW( value)
                out     dx,ax

                mov     dx,DSP32C_BaseIo
                add     dx,PDR
                mov     ax,HW( value)
                out     dx,ax

                }



        /* Restore the PCR. */

        WritePio( PCR, save);



}
FUNCTYPE( INT) DSP32C_SrHexLoad( unsigned long __far *input) {

        unsigned long dsp_start, nbytes;



        /* Download a DSPASM .dsh hex input array. */

        DSP32C_Halt();



        /* Check the leading array id. */

        if ( *input++ != 0x52535F5FL /* = "__SR */ )

                return( 0);



        /* Download records. */

        while ( *input != 0xFFFFFFFFL ) {

                dsp_start = *input++;
                   nbytes = *input++;

                   DSP32C_Put( (void __far *)input, dsp_start, nbytes);

                      ((char *)input) += nbytes;

                }



        /* Return success. */

        return( 1);


}










/* DSPMUL cpu selection:
 *
 * This function selects which cpu on a DSPMUL board is connected to
 * the PC bus.
 *
 * It MUST be called BEFORE DSP32C_Init to select which cpu will be
 * initialized.  After initialization of each cpu, it can be called as
 * needed to select which cpu will be used.
 *
 *
 */

#define BASESEL_ENV_NAME        "DSP32C_BASESEL"


FUNCTYPE( VOID) DSP32C_SelectCpu( unsigned n) {


        /* Initialize basesel.
         *
         * Note that this code allows the user to override either a DOS
         * environment space value or the normal default with a nonzero
         * value they have set before calling select_cpu.
         *
         * This can be useful for multiple board installations where the
         * user must call select_cpu with basesel set to the IO address
         * for each board.
         *
         *
         */

#define _SEARCHENV

#ifdef _SEARCHENV

        if ( DSP32C_BaseSel == 0x0000 ) {


                /* Check DOS environment space for an override. */

                DSP32C_BaseSel = getenv_DSP32C( BASESEL_ENV_NAME);



                /* If not found, use the default value. */

                if ( DSP32C_BaseSel == 0x0000 )

                        DSP32C_BaseSel = DEFAULT_BASESEL;

                }

#else
        DSP32C_BaseSel = DEFAULT_BASESEL;

#endif


        /* Set the DSPMUL cpu selection register. */

        _asm {

                mov     dx,DSP32C_BaseSel
                mov     ax,n
                out     dx,al

                }


}


