/* FILE: fft.dsp                Copyright (c), Symmetric Research, 1998
 *
 * This is the DSP32C portion of the fft program.  The basic algorithm
 * is the same as the function _fft in the AT&T/Lucent Software Library
 * Reference manual, but has been modified to use with the DSPASM
 * assembler.
 *
 * In C pseudo code, the source code for this program might read as:
 * 
 * fft( complex A[], int N) {
 *
 *      for ( i = j = 0 ; i < N ; i++ ) { <<< bit reverse A
 *
 *              if ( i < j ) {
 *
 *                      temp = A[i]; A[i] = A[j]; A[j] = temp;
 *
 *                      }
 *
 *              reverse_carry_inc( j);
 *
 *              }
 *
 *      for ( n = 2 ; n <= N ; n *= 2 ) { <<< compute butterflys
 *
 *              for ( k = 0 ; k < n/2 ; k++ ) {
 *
 *                      U = 1.0;
 *
 *                      for ( p = 0 ; p < N ; p += n ) {
 *
 *                              temp = A[k+p] + U*A[k+p+n/2];
 *                              A[k+p+n/2] = 2*A[k+p] - temp;
 *                              A[k+p] = temp;
 *
 *                              }
 *
 *                      U = U * cexp( 2*PI/n);
 *
 *                      }
 *      }
 * }
 *
 * where U and temp are complex quantities.
 *
 * Note that subtle use is make of the DAU pipeline latency in the code
 * below for the bit reversal and butterfly sections.  Temporary
 * variables are kept in the DAU pipeline and never explicitly
 * allocated.
 *
 * Finally, the incoming data is passed to this program in an integer
 * array.  This array is first unpacked into a complex floating point
 * array, then FFT'd, and converted back to a real integer array.
 *
 * Integer input and output is often required for plotting on the
 * screen.  This program shows how to roll both the integer conversions
 * and FFT into one operation.
 *
 *
 *
 */

constant

        PROGRAM         = 0x000000,
        DATA            = 0x001000;




<DATA>

long NArray; dspf Array[];




<PROGRAM>

main {

        /* Convert from IEEE to DSPF floats. */

        call IeeeToDspf;


        /* Compute the FFT. */

        call FFT;


        /* Convert back to IEEE floats. */

        call DspfToIeee;


        /* Signal the PC and wait. */

        r1 = 1; pir = r1;

        wait: goto wait;



}
DspfToIeee {

        register

                A1      = r1,
                A2      = r2,
                n       = r3;

        A1 = Array;
        A2 = Array;
         n = *NArray;

         while ( n-- > 0 ) {

                *A2++ = a0 = ieee( *A1++);   /* convert real part */
                *A2++ = a0 = ieee( *A1++);   /* convert imag part */

                }

        return;

}
IeeeToDspf {


        register

                A1      = r1,
                A2      = r2,
                n       = r3;

        A1 = Array;
        A2 = Array;
         n = *NArray;

         while ( n-- > 0 ) {

                *A2++ = a0 = dsp( *A1++);   /* convert real part */
                *A2++ = a0 = dsp( *A1++);   /* convert imag part */

                }


        return;


}

dspf /* FFT constants ... */

        ZERO    = 0.0,
        ONE     = 1.0,
        TWO     = 2.0,

        forward_twids[] = {     /* complex twiddle factors (cos,sin) */

                -1.0000000,  0.0000000,         /* PI           */
                 0.0000000, -1.0000000,         /* PI/2         */
                 0.7071068, -0.7071068,         /* PI/4         */
                 0.9238795, -0.3826834,         /* PI/8         */
                 0.9807853, -0.1950903,         /* PI/16        */
                 0.9951847, -0.0980171,         /* PI/32        */
                 0.9987955, -4.9067674E-2,      /* PI/64        */
                 0.9996988, -2.4541228E-2,      /* PI/128       */
                 0.9999247, -1.2271538E-2,      /* PI/256       */
                 0.9999812, -6.1358846E-3,      /* PI/512       */
                 0.9999953, -3.0679567E-3,      /* PI/1024      */
                 0.9999989, -1.5339801E-3,      /* PI/2048      */

                 };

FFT {

        /* Compute an FFT with dpsf floats as input and output. */

        register

                i               = r1,           /* bit reversal regs    */
                j               = r2,
                temp            = r3,
                m               = r4,

                zero            = r1,           /* butterfly regs       */
                one             = r2,
                two             = r3,
                twid            = r4,
                A1_start        = r5,
                A2_start        = r6,
                offset          = r7,
                A1              = r8,
                A2              = r9,
                Z1              = r10,
                Z2              = r11,

                n               = r15,
                k               = r16,
                p               = r17,
                inc             = r18,
                isign           = r19,
                size            = r20;



        /* Compute the in place bit reversal. */

        n = *NArray;

        A1_start = Array;  A1 = Array;  i = 0;
        A2_start = Array;  A2 = Array;  j = 0;

        m = n*4;

        while ( n-- > 0 ) {

                /* Swap A[i] and A[j] if not swapped yet. */

                if ( j > i ) { temp = A1;

                        *A1++ = a0 = *A2++;
                        *A1-- = a0 = *A2--;
                        *A2++ = a0 = *temp++;
                        *A2-- = a0 = *temp;

                        }

                /* Increment i and j, with reverse carry on j. */

                i += 8; j #= m;

                A1 = A1_start + i;
                A2 = A2_start + j;

                }



        /* Compute the butterflys. */

        twid = forward_twids;

           zero = ZERO;
            one = ONE;
            two = TWO;

             n = *NArray;
             p = n/2;
        offset = 8;

        for ( size = 2 ; size <= n ; size *= 2 ) {


                /* Initialize butterfly pointers and U = (a2,a3) = (1,0). */

                A1_start = Array;
                A2_start = A1_start + offset;

                a2 = *one;
                a3 = *zero;


                /* Loop on p and k. */

                k = size/2; while ( k-- > 0 ) {

                        A1 = A1_start; Z1 = A1;
                        A2 = A2_start; Z2 = A2;

                        inc = offset*2; inc -= 4;

                        p--; do p+1 { /* compute a butterfly */

                                         a0 = *A1++ + a2 * *A2++;
                                 *Z1++ = a0 =    a0 - a3 * *A2--;
                                         a1 = *A1-- + a3 * *A2++;
                                 *Z2++ = a0 =  - a0 + *two * *A1++;
                              *Z1++inc = a1 =    a1 + a2 * *A2++inc;
                              *Z2++inc = a1 =  - a1 + *two * *A1++inc;

                            } p++;


                        /* Inc A1 and A2 start, and twiddle U. */

                        A1_start += 8;
                        A2_start += 8;

                        a0 = a2 * *twid++; a2 = a0 - a3 * *twid;
                        a0 = a2 * *twid--; a3 = a0 + a3 * *twid;

                        }

                /* Loop to the next value of size. */

                p /= 2; offset *= 2; twid += 8;

                }


        return;
}
